/* ----------------------------------------------------------------------------
 *
 * project:		UT-SNMP
 *
 * filename:	crap.c
 *
 * purpose:		The program reads one or more .tree & .desc-files, generated
 *				by RAP and combines them in one big .tree & .desc-file
 *
 * author:		Martijn Visser
 *
 * ----------------------------------------------------------------------------
 */
 

/* constants */
#define ERROR -2
#define FAILED -1
#define OK 0

#define TREEEXT ".tree"
#define DESCEXT ".desc"
#define DEFAULT "foid"
#define TEMP "temp"

#define MAXLINE 80
#define MAXNAME 40
#define COMMENT "--"

#include <stdio.h>


/* typedefs */		
typedef struct files {
	FILE *tree;
	FILE *desc;
} files;


/* ----------------------------------------------------------------------------
 *
 * function:	openFiles
 *
 * arguments:	path (IN), path to files to open.
 *		mode (IN), mode of fopen().
 *		data(OUT), pointer to structure 'files'.
 *				This structure will contain the streams.
 *
 * return:	OK on succes and FAILED otherwise
 *		
 * purpose:	open .tree- & .desc-file that go with path
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	int 
openFiles(char *path, char *mode, files *data)
{	char *fileName;
	
	fileName = (char *) malloc(strlen(path)+strlen(TREEEXT));
	strcat(strcpy(fileName,path),TREEEXT);
	data->tree = fopen(fileName,mode);
	free(fileName);
	if (data->tree == NULL) {
		fprintf(stderr,"%s, %d: could not open \"%s\"...\n",__FILE__,__LINE__,fileName);
		return FAILED;
	}
	
	fileName = (char *) malloc(strlen(path)+strlen(TREEEXT));
	strcat(strcpy(fileName,path),DESCEXT);
	data->desc = fopen(fileName,mode);
	free(fileName);
	if (data->desc == NULL) {
		fprintf(stderr,"%s, %d: could not open \"%s\"...\n",__FILE__,__LINE__,fileName);
		return FAILED;
	}
	
	return OK;
}
	
/* ----------------------------------------------------------------------------
 *
 * function:	closeFiles
 *
 * arguments:	data (IN), structure 'files'.
 *
 * return:	-	
 *		
 * purpose:	close .desc- & .tree-file in data
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	void
closeFiles( files data )
{	fclose(data.tree);
	fclose(data.desc);
}

/* ----------------------------------------------------------------------------
 *
 * function:	swapFile
 *
 * arguments:	path1 (IN), path2 (IN), ext (IN)
 *
 * return:	-
 *		
 * purpose:	deletes [path1.ext] and renames [path2.ext] to [path1.ext]
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	void
swapFile( char *path1, char *path2, char *ext)
{	char *fileName1, *fileName2;
	fileName1 = (char *)malloc(strlen(path1)+strlen(ext));
	fileName2 = (char *) malloc(strlen(path2)+strlen(ext));
	strcat(strcpy(fileName1,path1),ext);
	strcat(strcpy(fileName2,path2),ext);
	remove(fileName1);
	rename(fileName2,fileName1);
	free(fileName1);
	free(fileName2);
}

/* ----------------------------------------------------------------------------
 *
 * function:	readLine
 *
 * arguments:	input (IN), pointer to input-stream.
 *		line (OUT), pointer to buffer to contain next line.
 *
 * return:	OK on succes, FAILED otherwise (EOF)
 *		
 * purpose:	reads a line from input, comments ("-- ...") are discarded
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	int
readLine( FILE *input, char *line)
{	while (fgets(line,MAXLINE,input) != NULL) {
	        if (strncmp(line,COMMENT,strlen(COMMENT)) == 0) continue;
	        return OK;
	}
	return FAILED;
}
	
/* ----------------------------------------------------------------------------
 *
 * function:	copyDesc
 *
 * arguments:	descNum (IN), label to give to description to be copied.
 *		srcDesc (IN), stream with source-description.
 *		destDesc (IN),stream with destination-description.
 *
 * return:	-
 *		
 * purpose:	copies one description from srcDesc- to destDesc-file, and 
 *		labels it descNum.
 *		a description starts and ends with a '"'
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	int
copyDesc( int descNum, FILE *srcDesc, FILE *destDesc)
{	char line[MAXLINE];
	char cSep;
	int nLoop, last=0;
	
	readLine(srcDesc,line);
	sscanf(line,"%*d\n");
	fprintf(destDesc,"%d\n",descNum);
	while (readLine(srcDesc,line) == OK) {
		
		fputs(line,destDesc);
		if (line[strlen(line)-2] == '\"') break;
	}
}

/* ----------------------------------------------------------------------------
 *
 * function:	copyEntry
 *
 * arguments:	src (IN), stream of source files.
 *		dest (IN),stream of destination files.
 *
 * return:	OK on succes, FAILED otherwise (EOF)
 *		
 * purpose:	copies one tree- (and if present one description-entry) 
 *		from src to dest .tree- & .desc-files
 *		
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
	int
copyEntry( struct files src, struct files dest)
{	static int descNum=0;
	char line[MAXLINE];
	char child[MAXNAME], parent[MAXNAME];
	int descPres;
	int nOldNum, nChild;
	
	if (readLine(src.tree,line) == OK) {
		descPres = sscanf(line,"%s%s%d%d",child,parent,&nChild,&nOldNum);
		fprintf(dest.tree,"%s\t%s %d",child,parent,nChild);
		if (descPres == 4) {
			fprintf(dest.tree,"\t%d",++descNum);
			copyDesc(descNum,src.desc,dest.desc);
		}
		fputc('\n',dest.tree);
		return OK;
	}
	return FAILED;
}


/* ----------------------------------------------------------------------------
 *
 * function:	main
 *
 * arguments:	argc (IN), argv (IN)
 *
 * return:	-
 *		
 * purpose:	* check arguments
 *		* if no arguments give usage, otherwise
 *			- open files
 *			- search for insert-postion
 *			- insert, parse
 *			- close files	
 *
 * remarks:
 *
 * ----------------------------------------------------------------------------
 */
void main( int argc, char **argv)
{	struct files append, old, new;
	char firstLine[MAXLINE], line[MAXLINE];
	char parent[MAXNAME], child[MAXNAME];
	int insertPos=0;
	
	
    system( "clear" );
    fprintf( stdout,
"
************************************************************************
*                                                                      *
* ut-crap (version %s)                                        *
* -------                                                              *
*                                                                      *
* This program requires two rfc input files. Both filenames must al-   *
* ready have an *.desc file and a *.tree file. For eaxmple, filename   *
* <rfc_file1> must have a <rfc_file1>.desc file and a<rfc_file1>.tree  *
* file. The same holds for <rfc_file2>.                                *
*                                                                      *
* Both *.desc files, <rfc_file1>.desc and <rfc_file2>.desc, will be    *
* concatenated and both *.tree files will be concatenated.             *
*                                                                      *
*                                              Success,                *
*                                                   snmp@cs.utwente.nl *
*                                                                      *
************************************************************************
", VERSION );

	/* check parameters */
	if (argc != 3) 
	{
		fprintf( stdout, "\n  usage: ut-crap <rcf-file1> <rfc-file2>\n");
		fprintf( stdout, "\n************************************************************************\n\n" );
		exit(FAILED);
	} 

	/* open files */
	if ((openFiles(argv[1],"rt",&append) == FAILED)
	    ||(openFiles((argc == 2)?DEFAULT:argv[2],"rt",&old) == FAILED)
	    ||(openFiles(TEMP,"wt",&new) == FAILED))
		exit(FAILED);
		
	/* scan first line for parent */
	if (readLine(append.tree,firstLine) == FAILED) {
		fprintf(stderr,"%s, %d: input-file empty...\n",__FILE__,__LINE__);
		exit(FAILED);
	}
	fprintf( stdout, "\n Status: " ); fflush( stdout );
	sscanf(firstLine,"%*s%s",parent);
		
	/* search for insert-position */
	while (readLine(old.tree,line) != FAILED) {
		sscanf(line,"%s",child);
		
		if (strcmp(parent,child) == 0) {
			insertPos = ftell(old.tree);
			break;
		}
	}
	rewind(old.tree);
	rewind(append.tree);
	
	/* copy up to insert-postion */
	while (ftell(old.tree) != insertPos) copyEntry(old,new);
	
	/* insert file */
	while (copyEntry(append,new) == OK) ;
	
	/* copy rest from insert-postion */
	while (copyEntry(old,new) == OK) ;

	/* remove old files, rename new ones */
	closeFiles(append);
	closeFiles(new);
	closeFiles(old);
	
	swapFile((argc == 2)?DEFAULT:argv[2],TEMP,TREEEXT);
	swapFile((argc == 2)?DEFAULT:argv[2],TEMP,DESCEXT);

	fprintf( stdout, " Ready !\n" ); fflush( stdout );
	fprintf( stdout, "\n************************************************************************\n\n" );
}
